# Django REST Framework HMAC Authentication

This library implements the [HMAC](https://en.wikipedia.org/wiki/HMAC) Authentication protocol for the [django-rest-framework](http://www.django-rest-framework.org/).  

This is useful because HMAC allows the client to authenticate and make an http request at the same time.

HMAC is used by the [botocore](https://github.com/boto/botocore) Python library by AWS and other places around the web.

# Requirements

- Python 3.6
- Django 2.0
- Django REST Framework 3.7

# Documentation

## How to use

#### Define a `View` with `HMACAuthentication` set in the `authentication_classes`

```python
from rest_framework.views import APIView
from rest_framework_hmac.authentication import HMACAuthentication

class MyView(APIView):
    authentication_classes = (HMACAuthentication,)
```

#### Register the `HMACKey` Signal, so a HMAC `key` and `secret` get generated each time a User is created.

```python
from django.conf import settings
from django.db.models.signals import post_save
from django.dispatch import receiver
from rest_framework_hmac.hmac_key.models import HMACKey

@receiver(post_save, sender=settings.AUTH_USER_MODEL)
def create_hmac_key(sender, instance=None, created=False, **kwargs):
    if created:
        HMACKey.objects.create(user=instance)
```

#### Use the above relationship between the Django `User` and `HMACKey` Model with the HMACClient to generate a request.

To successfully make a request to this View, the client must set 3 properties in the request header:

- Key
- Signature
- Timestamp

The `Key` is used to lookup the Django User

The `Signature` is the HMAC Signature generated by signing with the HMAC cryptographic key

The `Timestamp` can be any string timestamp. In the example HMACClient the timestamp is `ISO8601`

```python
from rest_framework_hmac.client import HMACClient
from django.auth.contrib.models import User

user = User.objects.create_user('bob')

hmac_client = HMACClient(user)

# the "request" should contain a `Timestamp` header because it
# will be used for signing. So, `request.META['Timestamp']`
signature = hmac_client.calc_signature(request)

# Make request with all required headers
import time
from rest_framework.test import APIClient
from rest_framework_hmac.tests.factory import ISO8601

client = APIClient()
headers = {
    'Key': user.hmac_key.key,
    'Signature': signature,
    'Timestamp': time.strftime(ISO8601, time.gmtime())
}
client.post('/notes/', {'title': 'new idea'}, format='json', **headers)
```

## HMACClient

`HMACClient` is a concrete class which implements it's own lookup for the HMAC cryptographic key. `HMACClient` is used in the above "how to use" documentation.

`BaseHMACClient` can be used if the programmer wants to implement thier own lookup
